==> v10.5.1.zip: brewman/overlord/src/app/ledger/ledger.component.ts <==
import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { MatAutocompleteSelectedEvent } from '@angular/material/autocomplete';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';
import { Observable, of as observableOf } from 'rxjs';
import { debounceTime, distinctUntilChanged, map, switchMap } from 'rxjs/operators';

import { Account } from '../core/account';
import { AccountService } from '../core/account.service';
import { ToCsvService } from '../shared/to-csv.service';

import { Ledger } from './ledger';
import { LedgerDataSource } from './ledger-datasource';
import { LedgerService } from './ledger.service';

@Component({
  selector: 'app-ledger',
  templateUrl: './ledger.component.html',
  styleUrls: ['./ledger.component.css'],
})
export class LedgerComponent implements OnInit, AfterViewInit {
  @ViewChild('accountElement', { static: true }) accountElement?: ElementRef;
  @ViewChild(MatPaginator, { static: true }) paginator?: MatPaginator;
  @ViewChild(MatSort, { static: true }) sort?: MatSort;
  info: Ledger = new Ledger();
  dataSource: LedgerDataSource = new LedgerDataSource(this.info.body);
  form: FormGroup<{
    startDate: FormControl<Date>;
    finishDate: FormControl<Date>;
    account: FormControl<Account | string | null>;
  }>;

  selectedRowId = '';
  debit = 0;
  credit = 0;
  running = 0;
  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = ['date', 'particulars', 'type', 'narration', 'debit', 'credit', 'running'];

  accounts: Observable<Account[]>;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private toCsv: ToCsvService,
    private ser: LedgerService,
    private accountSer: AccountService,
  ) {
    this.form = new FormGroup({
      startDate: new FormControl(new Date(), { nonNullable: true }),
      finishDate: new FormControl(new Date(), { nonNullable: true }),
      account: new FormControl<Account | string | null>(null),
    });

    this.accounts = this.form.controls.account.valueChanges.pipe(
      map((x) => ((x as Account).name !== undefined ? (x as Account).name : (x as string))),
      map((x) => (x !== null && x.length >= 1 ? x : null)),
      debounceTime(150),
      distinctUntilChanged(),
      switchMap((x) => (x === null ? observableOf([]) : this.accountSer.autocomplete(x))),
    );
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { info: Ledger };

      this.info = data.info;
      this.calculateTotals();
      this.form.setValue({
        account: this.info.account,
        startDate: moment(this.info.startDate, 'DD-MMM-YYYY').toDate(),
        finishDate: moment(this.info.finishDate, 'DD-MMM-YYYY').toDate(),
      });
      this.dataSource = new LedgerDataSource(this.info.body, this.paginator, this.sort);
    });
  }

  ngAfterViewInit() {
    setTimeout(() => {
      if (this.accountElement) {
        this.accountElement.nativeElement.focus();
      }
    }, 0);
  }

  displayFn(account?: Account): string {
    return account ? account.name : '';
  }

  calculateTotals() {
    this.debit = 0;
    this.credit = 0;
    this.running = 0;
    this.info.body.forEach((item) => {
      if (item.type !== 'Opening Balance') {
        this.debit += item.debit;
        this.credit += item.credit;
        if (item.posted) {
          this.running += item.debit - item.credit;
        }
      } else {
        this.running += item.debit - item.credit;
      }
      item.running = this.running;
    });
  }

  selected(event: MatAutocompleteSelectedEvent): void {
    this.info.account = event.option.value;
  }

  selectRow(id: string): void {
    this.selectedRowId = id;
  }

  show() {
    const info = this.getInfo();
    this.router.navigate(['ledger', info.account.id], {
      queryParams: {
        startDate: info.startDate,
        finishDate: info.finishDate,
      },
    });
  }

  getInfo(): Ledger {
    const formModel = this.form.value;

    return new Ledger({
      account: formModel.account as Account,
      startDate: moment(formModel.startDate).format('DD-MMM-YYYY'),
      finishDate: moment(formModel.finishDate).format('DD-MMM-YYYY'),
    });
  }

  exportCsv() {
    const headers = {
      Date: 'date',
      Name: 'name',
      Type: 'type',
      Narration: 'narration',
      Debit: 'debit',
      Credit: 'credit',
      Running: 'running',
      Posted: 'posted',
    };
    const csvData = new Blob([this.toCsv.toCsv(headers, this.dataSource.data)], {
      type: 'text/csv;charset=utf-8;',
    });
    const link = document.createElement('a');
    link.href = window.URL.createObjectURL(csvData);
    link.setAttribute('download', 'ledger.csv');
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }
}

==> v11.3.1.zip: brewman/overlord/src/app/ledger/ledger.component.ts <==
import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { MatAutocompleteSelectedEvent } from '@angular/material/autocomplete';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';
import { Observable, of as observableOf } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';

import { Account } from '../core/account';
import { AccountService } from '../core/account.service';
import { ToCsvService } from '../shared/to-csv.service';

import { Ledger } from './ledger';
import { LedgerDataSource } from './ledger-datasource';
import { LedgerService } from './ledger.service';

@Component({
  selector: 'app-ledger',
  templateUrl: './ledger.component.html',
  styleUrls: ['./ledger.component.css'],
})
export class LedgerComponent implements OnInit, AfterViewInit {
  @ViewChild('accountElement', { static: true }) accountElement?: ElementRef;
  @ViewChild(MatPaginator, { static: true }) paginator?: MatPaginator;
  @ViewChild(MatSort, { static: true }) sort?: MatSort;
  info: Ledger = new Ledger();
  dataSource: LedgerDataSource = new LedgerDataSource(this.info.body);
  form: FormGroup<{
    startDate: FormControl<Date>;
    finishDate: FormControl<Date>;
    account: FormControl<string | null>;
  }>;

  selectedRowId = '';
  debit = 0;
  credit = 0;
  running = 0;
  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = ['date', 'particulars', 'type', 'narration', 'debit', 'credit', 'running'];

  accounts: Observable<Account[]>;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private toCsv: ToCsvService,
    private ser: LedgerService,
    private accountSer: AccountService,
  ) {
    this.form = new FormGroup({
      startDate: new FormControl(new Date(), { nonNullable: true }),
      finishDate: new FormControl(new Date(), { nonNullable: true }),
      account: new FormControl<string | null>(null),
    });

    this.accounts = this.form.controls.account.valueChanges.pipe(
      debounceTime(150),
      distinctUntilChanged(),
      switchMap((x) => (x === null ? observableOf([]) : this.accountSer.autocomplete(x))),
    );
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { info: Ledger };

      this.info = data.info;
      this.calculateTotals();
      this.form.setValue({
        account: this.info.account?.name ?? '',
        startDate: moment(this.info.startDate, 'DD-MMM-YYYY').toDate(),
        finishDate: moment(this.info.finishDate, 'DD-MMM-YYYY').toDate(),
      });
      this.dataSource = new LedgerDataSource(this.info.body, this.paginator, this.sort);
    });
  }

  ngAfterViewInit() {
    setTimeout(() => {
      if (this.accountElement) {
        this.accountElement.nativeElement.focus();
      }
    }, 0);
  }

  displayFn(account?: Account | string): string {
    return !account ? '' : typeof account === 'string' ? account : account.name;
  }

  calculateTotals() {
    this.debit = 0;
    this.credit = 0;
    this.running = 0;
    this.info.body.forEach((item) => {
      if (item.type !== 'Opening Balance') {
        this.debit += +item.debit;
        this.credit += +item.credit;
        if (item.posted) {
          this.running += +item.debit - +item.credit;
        }
      } else {
        this.running += +item.debit - +item.credit;
      }
      item.running = this.running;
    });
  }

  selected(event: MatAutocompleteSelectedEvent): void {
    this.info.account = event.option.value as Account;
  }

  selectRow(id: string): void {
    this.selectedRowId = id;
  }

  show() {
    const info = this.getInfo();
    if (info.account) {
      this.router.navigate(['ledger', info.account.id], {
        queryParams: {
          startDate: info.startDate,
          finishDate: info.finishDate,
        },
      });
    }
  }

  getInfo(): Ledger {
    const formModel = this.form.value;

    return new Ledger({
      account: this.info.account,
      startDate: moment(formModel.startDate).format('DD-MMM-YYYY'),
      finishDate: moment(formModel.finishDate).format('DD-MMM-YYYY'),
    });
  }

  exportCsv() {
    const headers = {
      Date: 'date',
      Name: 'name',
      Type: 'type',
      Narration: 'narration',
      Debit: 'debit',
      Credit: 'credit',
      Running: 'running',
      Posted: 'posted',
    };
    const csvData = new Blob([this.toCsv.toCsv(headers, this.dataSource.data)], {
      type: 'text/csv;charset=utf-8;',
    });
    const link = document.createElement('a');
    link.href = window.URL.createObjectURL(csvData);
    link.setAttribute('download', 'ledger.csv');
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }
}
